#!/usr/bin/python

# Image Merge - For Creating Spritesheets - v0.2
# Copyright (C) 2006 Chris 'fydo' Hopp - September / October 2006
# Web: fydo.net	E-Mail: fydo@fydo.net

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# A copy of the GPL is available at http://www.gnu.org/copyleft/gpl.html
# and should also be included as gpl.txt

#
#	Requirements:
#		Python
#		PyGame >=1.8	OR	PIL
#
# The PIL portion of the script also requires a blank .png file (blank.png)
#  to be present as well.
#
# Pass in any number of image files and this script will merge them (from 
#  left to right in the order they were passed in) into one large, wide
#  image. 
#
# It is important to note that all of the images passed in must
#  be of the same height. Differing widths is supported by this script.
#
# Special Thanks to the following people for helping me with testing:
#	coderanger
#	RoeBros[0]
#

# Import the needed libraries
import os, sys

# First we shall handle the arguments
if len(sys.argv) < 2:
	print 'Incorrect usage. Try --help for some more information.'
	sys.exit()

# Now we'll look for anyone looking for help or version numbers
if sys.argv[1].startswith('--'):
	option = sys.argv[1][2:]
	if option == 'version':
		print 'Image Merge (merge.py) - by Chris Hopp - v0.2'
	elif option == 'help':
		print '''\
Image Merge (merge.py) - by Chris Hopp - v0.2

This script will build a spritesheet based on .png files passed into it.
Any number of .png files may be specified.

Example of usage:
	merge.py one.png two.png three.png

Additional options:
	--version	: Displays the version number
	--help		: Displays this utterly helpful help message
'''
	else:
		print 'Unknown option.'
	sys.exit()

# Now check to ensure that all of the files actually exist
for arg in sys.argv[1:]:
	if not os.path.exists(arg):
		print 'The file', arg, 'does not exist. Exiting ...'
		sys.exit()

usePygame = True # Default

# Attempt to use PyGame, but only >=v1.8 is allowed.
try:
	import pygame
	if (pygame.version.vernum < (1, 8)):
		print 'Your version of PyGame (%s) doesn\'t support png saving (Needs >=v1.8). Trying PIL ...' % pygame.version.ver
		usePygame = False
except ImportError:
	print 'PyGame not found. Trying PIL ...'
	usePygame = False

# No usable PyGame, so lets try to use PIL.
if usePygame == False:
	try:
		from PIL import Image
		# PIL requires the blank.png file to exist
		if not os.path.exists('blank.png'):
			print 'PIL version of this script requires \'blank.png\'. Exiting ...'
			sys.exit()
		else:
			print 'Success! Using PIL...'
	except ImportError:
		print 'No suitable image modification libraries found. Exiting ...'
		sys.exit()

def main(argv):

	global usePygame

	# Initialize stuff ...
	neww = 0
	newh = 0
	count = 0
	sub_images = []
	print ' ' # Just to make things look a little nicer :)

	if usePygame: # Using PyGame version 1.8 or higher

		# Allow us to play around with pixel data without actually opening a window
		os.environ['SDL_VIDEODRIVER'] = 'dummy'

		# Start things up
		pygame.init()
		pygame.display.set_mode((1,1))

		# Gather data about each individual image file
		for arg in argv:
	
			# Show everyone we're currently processing this file
			print "loading: " + arg
		
			# Open the file
			try:
				sub_images.append(pygame.image.load(arg))
			except:
				print "Error loading file", arg
				return

			sub_images[count].convert_alpha()
	
			# Grab the dimensions		
			neww += sub_images[count].get_width()
			newh = sub_images[count].get_height()
	
			# Incrementation!
			count += 1
	
		# Set up the pygame surface
		im = pygame.Surface((neww, newh)).convert_alpha()
		# Set the pixel alpha (NOT surface alpha)
		pygame.surfarray.pixels_alpha(im)[:,:] = 0
	
		# Loop through all the images and blit them to the surface
		for y in range(count):
			im.blit(sub_images[y], (y * sub_images[y].get_width(), 0))
	
		# All done! Saving time!
		pygame.image.save(im, "out.png")
	
	else: # Using PIL

		# Gather data about each individual image file
		for arg in argv:

			# Show everyone we're currently processing this file
			print "loading: " + arg
			
			# Open the file
			try:
				sub_images.append(Image.open(arg))
			except:
				print "Error loading file", arg
				return
	
			# Grab the dimensions		
			neww += sub_images[count].size[0]
			newh = sub_images[count].size[1]
	
			# Incrementation!
			count += 1
	
		# Load that blank.png file
		try:
			im = Image.open("blank.png")
		except:
			print "Error loading file blank.png"
			return

		# Resize that blank.png file
		im = im.resize((neww, newh))
	
		# Loop through all the images and append them to that blank.png file
		for y in range(count):
			im.paste(sub_images[y], (y * sub_images[y].size[0], 0))
	
		# All done! Saving time!
		im.save("out.png")

	# Report to the user and seek approval
	print '\nDone. Merged', count, 'images. File saved as out.png. Press return.'
	a = raw_input()
	
	
if __name__ == "__main__":
	main(sys.argv[1:]) # Don't want the very first arg, which is the executable name